"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const path_1 = __importStar(require("path"));
const logger_1 = __importDefault(require("../logger"));
const tool_1 = require("../tool");
class GqlLoader {
    constructor() {
        this.resolversPath = path_1.join(tool_1.PathsResolverTool.BUILD, 'resolvers');
        this.resolvers = {};
        this.getResolverName = (file) => path_1.default.parse(file).name;
    }
    async loadResolversFromDir(folder) {
        const normalizedPath = path_1.default.join(this.resolversPath, folder);
        let files;
        try {
            files = await tool_1.PromisifiedFs.readdir(normalizedPath);
            files = files.filter((file) => /.js$/.test(file));
        }
        catch (e) {
            if ((e === null || e === void 0 ? void 0 : e.code) !== 'ENOENT') {
                logger_1.default.error(e, undefined, this.constructor.name);
            }
            return null;
        }
        const resolversArray = await Promise.all(files.map(async (it) => ({
            [this.getResolverName(it)]: await tool_1.importUncached(path_1.default.join(normalizedPath, it)),
        })));
        const reducedResolversObject = resolversArray.reduce((prev, next) => {
            const name = Object.keys(next)[0];
            const value = Object.values(next)[0];
            if (!value.default) {
                return Object.assign({}, prev);
            }
            return Object.assign(Object.assign({}, prev), { [name]: value.default });
        }, {});
        if (Object.keys(reducedResolversObject).length) {
            return reducedResolversObject;
        }
        return null;
    }
    async loadResolversFromFile() {
        const resolversFile = await tool_1.importUncached(path_1.default.join(this.resolversPath, 'resolvers.js'));
        Object.entries(resolversFile.default)
            .filter(([key]) => key !== 'Query' && key !== 'Mutation')
            .forEach(([key, value]) => {
            this.resolvers[key] = value;
        });
        this.resolvers.Query = Object.assign(Object.assign({}, this.resolvers.Query), resolversFile.default.Query);
        if (Object.keys(resolversFile.default.Mutation).length) {
            this.resolvers.Mutation = Object.assign(Object.assign({}, this.resolvers.Mutation), resolversFile.default.Mutation);
        }
    }
    async loadResolvers() {
        const query = await this.loadResolversFromDir('queries');
        const mutation = await this.loadResolversFromDir('mutations');
        if (query) {
            this.resolvers.Query = query;
        }
        if (mutation) {
            this.resolvers.Mutation = mutation;
        }
        await this.loadResolversFromFile();
    }
    async getResolvers() {
        await this.loadResolvers();
        const context = this.constructor.name;
        logger_1.default.log(`Loaded queries: ${Object.keys(this.resolvers.Query)}`, context);
        if (this.resolvers.Mutation) {
            logger_1.default.log(`Loaded mutations: ${Object.keys(this.resolvers.Mutation)}`, context);
        }
        return this.resolvers;
    }
    async getResolversInArray() {
        await this.loadResolvers();
        const { Query: query, Mutation: mutation } = this.resolvers;
        const queryResolvers = query ? Object.keys(query) : undefined;
        const mutationResolvers = mutation ? Object.keys(mutation) : undefined;
        return {
            queryResolvers,
            mutationResolvers,
        };
    }
    static async getTypeDefs() {
        return tool_1.PromisifiedFs.readFile(tool_1.PathsResolverTool.SCHEMA, 'utf8');
    }
    static async importOrNull(resolverPath) {
        if (await tool_1.PathsResolverTool.resolvePath(resolverPath)) {
            return tool_1.importUncached(resolverPath).then((result) => result.default);
        }
        return null;
    }
    async getContextHandler() {
        return GqlLoader.importOrNull(path_1.join(this.resolversPath, 'context.handler.js'));
    }
    async getAuthHandler() {
        return GqlLoader.importOrNull(path_1.join(this.resolversPath, 'auth.handler.js'));
    }
    customScalars(schema) {
        const paginationAmount = { min: 1, max: 100 };
        function validatePagination(value) {
            const int = Number.parseInt(value, 10);
            if (!int || int < paginationAmount.min || int > paginationAmount.max) {
                throw new Error(`Expected ${value} to be integer between ${paginationAmount.min} and ${paginationAmount.max}`);
            }
            return int;
        }
        const typeConfigs = [{
                name: 'PaginationAmount',
                description: `Int between ${paginationAmount.min} and ${paginationAmount.max} used for paginating`,
                serialize: validatePagination,
                parseValue: validatePagination,
                parseLiteral(ast) {
                    if (ast.kind === graphql_1.Kind.INT) {
                        return validatePagination(ast.value);
                    }
                    return null;
                },
            },
        ];
        const result = {};
        typeConfigs.forEach((typeConfig) => {
            if (schema.includes(typeConfig.name))
                result[typeConfig.name] = new graphql_1.GraphQLScalarType(typeConfig);
        });
        return result;
    }
}
exports.default = GqlLoader;
//# sourceMappingURL=gql.loader.js.map