"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const bodyParser = __importStar(require("body-parser"));
const path_1 = require("path");
const tool_1 = require("../tool");
const logger_1 = __importDefault(require("../logger"));
const rest_handler_1 = __importDefault(require("./rest.utils/rest-handler"));
const tools_1 = __importDefault(require("./rest.utils/tools"));
const middleware_pool_1 = __importDefault(require("./rest.utils/middleware-pool"));
class RestLoader {
    constructor(app, ctx, restConfig) {
        this.app = app;
        this.ctx = ctx;
        this.restConfig = restConfig;
        this.handlersPath = path_1.join(tool_1.PathsResolverTool.BUILD, 'rest');
    }
    async loadEndpointsUsingYaml() {
        if (!this.restConfig || !this.restConfig.routes) {
            return [];
        }
        return this.restConfig.routes
            .map((it) => {
            const { method, path, handler, requestMiddlewares, responseMiddlewares, } = it;
            return new rest_handler_1.default(method, path, path_1.join(tool_1.PathsResolverTool.BUILD, handler), {
                request: requestMiddlewares || [],
                response: responseMiddlewares || [],
            });
        });
    }
    async loadEndpointUsingFolderStructure(yamlEndpoints) {
        try {
            await tool_1.PromisifiedFs.access(this.handlersPath);
        }
        catch (e) {
            return [];
        }
        const files = await tool_1.PromisifiedFs.readdir(this.handlersPath);
        return files
            .filter((path) => (/\.[tj]s$/g).test(path))
            .map(tools_1.default)
            .map((file) => new rest_handler_1.default(file.method, file.route, `${this.handlersPath}/${file.path}`))
            .filter((handler) => yamlEndpoints.every((existedHandler) => existedHandler.identifier !== handler.identifier));
    }
    async loadEndpoints() {
        const handlersFromYaml = await this.loadEndpointsUsingYaml();
        const handlersFromStructure = await this.loadEndpointUsingFolderStructure(handlersFromYaml);
        this.handlers = [...handlersFromYaml, ...handlersFromStructure];
        this.middlewarePool = await new middleware_pool_1.default(this.handlers, this.ctx).load();
        await this.applyToApplication();
        return this.handlers.map((handler) => handler.routeInfo);
    }
    async applyToApplication() {
        const router = express_1.Router();
        logger_1.default.log('Loaded REST handlers:', this.constructor.name);
        this.handlers.map(async (handler) => {
            const { route, method } = handler.routeInfo;
            if (!await handler.isExists()) {
                return;
            }
            const { request, response } = this.middlewarePool.getGroupedMiddlewares(handler);
            const wrappedHandler = await handler.importAndWrap(this.ctx);
            logger_1.default.log(`-> ${route} - ${method.toUpperCase()}`, this.constructor.name);
            router[method](route, ...request, wrappedHandler, ...response);
        });
        this.app.use('/rest', bodyParser.json(), router);
    }
}
exports.default = RestLoader;
//# sourceMappingURL=rest.loader.js.map