"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tool_1 = require("../../tool");
class RestHandler {
    constructor(method, route, pathToFile, middlewares = {
        request: [],
        response: [],
    }) {
        this.method = method;
        this.route = route;
        this.pathToFile = pathToFile;
        this.middlewares = middlewares;
        this.method = method.toLowerCase();
        this.identifier = method + route;
        this.pathToFile = (/\.[tj]s$/g).test(pathToFile) ? pathToFile : `${pathToFile}.js`;
    }
    async importAndWrap(ctx) {
        const { default: handler } = await tool_1.importUncached(this.pathToFile);
        return this.wrap(handler, ctx);
    }
    wrap(handler, ctx) {
        const handlerResolver = async (req, res, next) => {
            let shouldCallNext = false;
            const nextFunc = () => {
                shouldCallNext = true;
            };
            const { body, headers, query } = req;
            const event = {
                params: {
                    query,
                },
                body,
                headers,
                req,
                res,
                next: nextFunc,
            };
            try {
                const result = await handler(event, ctx);
                if (shouldCallNext) {
                    return next();
                }
                if (typeof result === 'object') {
                    if (result.status) {
                        return res.status(result.status).send(result.data);
                    }
                    return res.status(200).send(result.data);
                }
                return res.status(200).send(result);
            }
            catch (e) {
                return res.status(500).send(e);
            }
        };
        return handlerResolver;
    }
    get routeInfo() {
        return {
            method: this.method,
            route: this.route,
        };
    }
    async isExists() {
        try {
            await tool_1.PromisifiedFs.access(this.pathToFile);
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
exports.default = RestHandler;
//# sourceMappingURL=rest-handler.js.map