"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
class RequestCostTool {
    constructor(redis) {
        this.redis = redis;
        this.COST_EXPIRATION_IN_SECONDS = 3600;
        this.costTable = 'queryCost';
        this.context = 'Redis';
        if (!redis) {
            this.redis = new Proxy({}, {
                get() {
                    return () => undefined;
                },
            });
        }
    }
    setCost(request, cost) {
        const result = request;
        const { body, ip } = request;
        body.query = body.query.replace(/\s+/g, ' ');
        const costData = {
            id: uuid_1.v4(),
            ip,
            cost,
            body,
        };
        Object.freeze(costData);
        Object.defineProperty(result, 'costData', {
            value: costData,
            writable: false,
            configurable: false,
        });
        return result;
    }
    async saveCost(request) {
        const costData = this.getCostDataFromRequest(request);
        if (!costData)
            return;
        const key = this.tableCostForUser(costData.ip, costData.id);
        await this.redis.set(key, JSON.stringify(costData));
        await this.redis.expire(key, this.COST_EXPIRATION_IN_SECONDS);
    }
    async getTotalCost(request) {
        const costData = this.getCostDataFromRequest(request);
        if (!costData)
            return 0;
        const keys = await this.redis.keys(this.tableCostForUser(costData.ip, '*'));
        if (!(keys === null || keys === void 0 ? void 0 : keys.length))
            return 0;
        const costs = await this.redis.mget(keys);
        if (!(costs === null || costs === void 0 ? void 0 : costs.length))
            return 0;
        return costs.map((costJson) => JSON.parse(costJson))
            .reduce((total, { cost }) => total + cost, 0);
    }
    tableCostForUser(ip, id) {
        return `${this.costTable}:${ip}:${id}`;
    }
    requestHasCostGuard(request) {
        return Boolean(request.costData);
    }
    getCostDataFromRequest(request) {
        if (!this.requestHasCostGuard(request)) {
            return null;
        }
        return request.costData;
    }
}
exports.default = RequestCostTool;
//# sourceMappingURL=request-cost.tool.js.map